(* nstring - extra string functions for vpidentd
   
   (C) 2002 Nicholas Jay Kain <niklata@aerifal.cx>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation. 

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)

(* returns everything to the left of first occurence of char c *)
let lgrab str c = 
    let finish = String.index str c in
      String.sub str 0 finish
;;

(* returns everything to the right of first occurence of char c *)
let rgrab str c = 
    let start = String.index str c + 1 in
    let finish = String.length str - start in
      String.sub str start finish
;;

(* returns everything to the left of character n *)
let lgrabn str n =
  let finish = String.length str in
    if n < 0 then "" else
      if n <= finish then String.sub str 0 n else str
;;

(* returns everything to the right of character n *)
let rgrabn str n =
  let finish = String.length str in
    if n < 0 then str else
      if n <= finish then String.sub str n (finish - n) else ""
;;

(* removes all spaces on the left side of str *)
let rec ltrim str =
  try
    let t = rgrab str ' ' in
    let delta = String.length str - String.length t in
      if delta > 1 then str else ltrim t
  with Not_found -> str
;;

(* removes all spaces on the right side of str *)
let rec rtrim str = 
  try
    let t = lgrab str ' ' in
    let delta = String.length str - String.length t in
      if delta > 1 then str else rtrim t
  with Not_found -> str
;;

(* removes all spaces on both sides of str *)
let trim str = 
  rtrim (ltrim str)
;;

let filter_non_numerics str =
  let out = ref "" and len = String.length str in
    for i = 0 to len - 1 do
      match str.[i] with
	  '0'..'9' as t -> out := !out ^ String.make 1 t
	| _ -> ()
    done;
    !out
;;

exception Found_non_numeric ;;

let extract_first_numeric str =
  let out = ref "" and len = String.length str in
    try
      for i = 0 to len - 1 do
	match str.[i] with
	    '0'..'9' as t -> out := !out ^ String.make 1 t
	  | _ -> raise Found_non_numeric
      done;
      !out
    with Found_non_numeric -> !out
;;

exception Found_substr ;;

let contains_substr str sub =
  try
    let sublen = String.length sub in
      while true do
	let loc = String.index str sub.[0] in
	  if (String.sub str loc sublen) = sub then raise Found_substr
      done ;
      false
  with
      Found_substr -> true
    | _ -> false
;;
