(* printcfg - pretty prints user's .ident
   
   (C) 2002 Nicholas Jay Kain <niklata@aerifal.cx>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation. 

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *)


open Parsecfg

let rec print_i_list t = 
let rec print_hl_list = function
    [] -> ()
  | IP_ADDR :: r -> print_string "ip " ; print_hl_list r
  | S_PORT :: r -> print_string "sp " ; print_hl_list r
  | D_PORT :: r -> print_string "dp " ; print_hl_list r
  | UID :: r -> print_string "uid " ; print_hl_list r
in
  match t with
      [] -> ()
    | { ip=i; mask=m; sports=(sl,sh); dports=(dl,dh); response=r; } :: rem ->
	print_string "IP: " ;
	print_string (Unix.string_of_inet_addr i) ;
	print_string "\nMask: " ;
	print_string (Unix.string_of_inet_addr m) ;
	print_string "\nLocal Ports: " ;
	print_int sl ; print_string " to " ; print_int sh ;
	print_string "\nDestination Ports: " ;
	print_int dl ; print_string " to " ; print_int dh ;
	print_string "\nResponse: " ;
	begin
	  match r with
	      Deny -> print_string "Deny"
	    | Accept -> print_string "Accept"
	    | Spoof s -> print_string ("Spoof as \"" ^ s ^ "\"")
	    | Hash hl -> 
		begin
		  print_string "Hash with " ; 
		  print_hl_list hl
		end
	end ;
	print_string "\n\n" ;
	print_i_list rem
;;

let print_file filename =
  let items = read_file filename in
    print_i_list items
;;

print_file "/home/njk/.ident"
