(* proctcp - encapsulates linux 2.4 /proc/net/tcp data
   
   (C) 2002 Nicholas Jay Kain <niklata@aerifal.cx>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation. 

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)

open Nstring ;;

let proc_net_tcp = "/proc/net/tcp"
;;

exception Bad_ip_length ;;

type pnt_query_response = 
    Error
  | User of int
;;

type tcp_entry = {
  s_ip: string;
  s_port: int;
  d_ip: string;
  d_port: int;
  uid: int;
}
;;

let decode_ip ip =
  if String.length ip <> 8 then raise Bad_ip_length else
    let a = "0x" ^ String.sub ip 0 2 and b = "0x" ^ String.sub ip 2 2 and
        c = "0x" ^ String.sub ip 4 2 and d = "0x" ^ String.sub ip 6 2 in
    let a = int_of_string a and b = int_of_string b and c = int_of_string c and
        d = int_of_string d in
    let a = string_of_int a and b = string_of_int b and c = string_of_int c and
        d = string_of_int d in
      d ^ "." ^ c ^ "." ^ b ^ "." ^ a (* assumes little endian *)
;;

let decode_parse sie spe die dpe uide = 
  let u = int_of_string uide in
  let dp = int_of_string ("0x" ^ dpe) in
  let sp = int_of_string ("0x" ^ spe) in
  let di = decode_ip die in
  let si = decode_ip sie in
    { s_ip = si ; s_port = sp ; d_ip = di ; d_port = dp ; uid = u }
;;

(* this is rather ugly... I would use a lexer, except it's overkill *)
let parse_pnt in_line =
  let line = ref "" and sie = ref "" and die = ref "" and spe = ref "" and
    dpe = ref "" and uide = ref "" in
  line := rgrab (ltrim in_line) ' ' ;

  sie := ltrim (lgrab !line ':') ;
  line := rgrab !line ':' ;

  spe := lgrab !line ' ' ;
  line := rgrab !line ' ' ;

  die := lgrab !line ':' ;
  line := rgrab !line ':' ;

  dpe := lgrab !line ' ' ;
  line := rgrab !line ' ' ;
  
  line := rgrab !line ':' ;
  line := rgrab !line ':' ;
  line := rgrab !line ' ' ;
  line := ltrim (rgrab !line ' ') ;
  
  uide := lgrab !line ' ' ;
    decode_parse !sie !spe !die !dpe !uide
;;

let read_pnt file = 
  let input = ref ([] : tcp_entry list) in
  let fh = open_in file in
    try
        ignore (input_line fh) ;
 	while true do
	  let l = input_line fh in
	  let t = parse_pnt l in
	    input := t :: !input
 	done;
      !input
    with
	End_of_file -> close_in fh ; !input
;;

let rec query_list l host sp dp =
  match l with
      [] -> Error
    | h::t -> if h.d_ip = host && h.s_port = sp && h.d_port = dp
      then User h.uid
      else query_list t host sp dp
;;

let query host sp dp =
  query_list (read_pnt proc_net_tcp) host sp dp
;;
